@@ -11,11 +11,13 @@
 
 local log = require "util.logger".init("rostermanager");
 
+local new_id = require "util.id".short;
+
 local pairs = pairs;
 local tostring = tostring;
 local type = type;
 
-local hosts = hosts;
+local hosts = prosody.hosts;
 local bare_sessions = prosody.bare_sessions;
 
 local um_user_exists = require "core.usermanager".user_exists;
@@ -23,6 +25,7 @@ local st = require "util.stanza";
 local storagemanager = require "core.storagemanager";
 
 local _ENV = nil;
+-- luacheck: std none
 
 local save_roster; -- forward declaration
 
@@ -60,7 +63,7 @@ local function roster_push(username, host, jid)
 	local roster = jid and hosts[host] and hosts[host].sessions[username] and hosts[host].sessions[username].roster;
 	if roster then
 		local item = hosts[host].sessions[username].roster[jid];
-		local stanza = st.iq({type="set"});
+		local stanza = st.iq({type="set", id=new_id()});
 		stanza:tag("query", {xmlns = "jabber:iq:roster", ver = tostring(roster[false].version or "1")  });
 		if item then
 			stanza:tag("item", {jid = jid, subscription = item.subscription, name = item.name, ask = item.ask});
@@ -104,18 +107,29 @@ local function load_roster(username, host)
 	if user then
 		roster = user.roster;
 		if roster then return roster; end
-		log("debug", "load_roster: loading for new user: %s@%s", username, host);
+		log("debug", "load_roster: loading for new user: %s", jid);
 	else -- Attempt to load roster for non-loaded user
-		log("debug", "load_roster: loading for offline user: %s@%s", username, host);
+		log("debug", "load_roster: loading for offline user: %s", jid);
 	end
 	local roster_store = storagemanager.open(host, "roster", "keyval");
 	local data, err = roster_store:get(username);
 	roster = data or {};
 	if user then user.roster = roster; end
+	local legacy_pending = roster.pending and type(roster.pending.subscription) ~= "string";
 	roster_metadata(roster, err);
+	if legacy_pending then
+		-- Due to map store use, we need to manually delete this entry
+		log("debug", "Removing legacy 'pending' entry");
+		if not save_roster(username, host, roster, "pending") then
+			log("warn", "Could not remove legacy 'pendig' entry");
+		end
+	end
 	if roster[jid] then
 		roster[jid] = nil;
-		log("warn", "roster for %s has a self-contact", jid);
+		log("debug", "Roster for %s had a self-contact, removing", jid);
+		if not save_roster(username, host, roster, jid) then
+			log("warn", "Could not remove self-contact from roster for %s", jid);
+		end
 	end
 	if not err then
 		hosts[host].events.fire_event("roster-load", { username = username, host = host, roster = roster });
