@@ -79,9 +79,11 @@ static void base64_decode(luaL_Buffer *b, int c1, int c2, int c3, int c4, int n)
 	switch(--n) {
 		case 3:
 			s[2] = (char) tuple;
+			/* Falls through. */
 
 		case 2:
 			s[1] = (char)(tuple >> 8);
+			/* Falls through. */
 
 		case 1:
 			s[0] = (char)(tuple >> 16);
@@ -216,7 +218,7 @@ static const char *utf8_decode(const char *o, int *val) {
  */
 const char *check_utf8(lua_State *L, int idx, size_t *l) {
 	size_t pos, len;
-	const char *s = luaL_checklstring(L, 1, &len);
+	const char *s = luaL_checklstring(L, idx, &len);
 	pos = 0;
 
 	while(pos <= len) {
